# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import


class BgColors(object):
    """ Background Colors
    """
    HEADER = '\033[95m'
    BLUE = '\033[94m'
    YELLOW = '\033[93m'
    GREEN = '\033[92m'
    WARNING = '\033[93m'
    RED = '\033[91m'
    GRAY = '\033[90m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    COLORS = (BLUE, BOLD, GRAY, GREEN, RED, YELLOW)

    def __init__(self):
        raise NotImplementedError


class Daemons(object):
    """ Daemons running on the system
    """
    BGPD = 'bgpd'
    CLAG = 'clagd'
    LLDP = 'lldpd'
    MISC = 'misc'
    MSTPD = 'mstpd'
    OSPFD = 'ospfd'
    OSPF6D = 'ospf6d'
    ZEBRA = 'zebra'

    def __init__(self):
        raise NotImplementedError


class DefaultPaths(object):

    CONF_FILE = '/etc/netq/netq.yml'
    CONF_DIR = '/etc/netq/config.d/'
    DBCONFIG = '%snetq-agent-commands.yml' % CONF_DIR
    DBCONFIG_RUNNING = '/var/run/netq-agent-running.json'
    INTERFACE_FILE = '/etc/network/interfaces'
    UDS_FILE = '/var/run/netq-agent.sock'

    def __init__(self):
        raise NotImplementedError


class ConfKw(object):
    '''Keywords for the fields in user config'''
    BACKEND = 'backend'
    SERVER = 'server'
    PORT = 'port'
    VRF = 'vrf'
    NETQD = 'netqd'
    HA = 'ha'
    ROLE = 'role'
    MASTER = 'master'
    SENTINEL = 'sentinel'
    REPLICA = 'replica'

    def __init__(self):
        raise NotImplementedError


class DiffState(object):

    ADDED = 0
    MODIFIED = 1
    DELETED = 2

    MAP = {
        ADDED: 'Addded',
        MODIFIED: 'Modified',
        DELETED: 'Deleted'
    }

    def __init__(self):
        raise NotImplementedError


class Heartbeat(object):
    """ These are shared across netq-agent and netq
    """
    LIVELINESS_RATE = 30
    HEARTBEAT_DEATH = 3 * LIVELINESS_RATE

    def __init__(self):
        raise NotImplementedError


class InterfaceState(object):
    """ Interface state
    """
    UP = 'up'
    DOWN = 'down'

    def __init__(self):
        raise NotImplementedError


class InterfaceDownReason(object):
    '''Reasons why an interface is down'''

    PROTODOWN = 'protodown'
    NO_CARRIER = 'no_carrier'
    DORMANT = 'dormant'
    NONE = ''

    def __init__(self):
        raise NotImplementedError


class LinkType(object):
    """Link types such as vxlan, bridge etc.
    """
    BCM = 'bcm_knet'
    BOND = 'bond'
    BRIDGE = 'bridge'
    ETH = 'eth'
    IGNORE = 'ignore'
    LOOPBACK = 'loopback'
    MACVLAN = 'macvlan'
    SWP = 'swp'
    TUN = 'tun'
    VLAN = 'vlan'
    VXLAN = 'vxlan'
    VRF = 'vrf'


class NeighborState(object):
    """ IP Neighbor state
    """
    UPDATE = 1
    DELETE = 2
    IGNORE = 3

    def __init__(self):
        raise NotImplementedError


class NetDevFmt(object):

    IFACE = 0
    RX_BYTES = 1
    RX_PKTS = 2
    RX_ERR = 3
    RX_DRP = 4
    RX_FIFO = 5
    RX_FRAME = 6
    RX_COMP = 7
    RX_MCAST = 8
    TX_BYTES = 9
    TX_PKTS = 10
    TX_ERR = 11
    TX_DRP = 12
    TX_FIFO = 13
    TX_COLL = 14
    TX_CARR = 15
    TX_COMP = 16

    def __init__(self):
        raise NotImplementedError


class Platform(object):
    """ Platform
    """
    CUMULUS = 'cumulus'
    LINUX = 'linux'
    UNKNOWN = 'UNKNOWN'
    VX = 'vx'
    CL = 'Cumulus Linux'


class Protocol(object):
    """ Protocols
    """
    KERNEL = 'kernel'
    ZEBRA = 'zebra'
    STATIC = 'static'
    BOOT = 'boot'
    BGP = 'bgp'
    OSPF = 'ospf'
    ISIS = 'is-is'

    def __init__(self):
        raise NotImplementedError


class RedisKeyType(object):
    """Redis Key Types
    """
    HASH = 'hash'
    LIST = 'list'
    SET = 'set'
    ZSET = 'zset'
    STRING = 'string'

    def __init__(self):
        raise NotImplementedError


class RunningService(object):
    """Service name
    """
    ACLINIT = 'aclinit'
    ACLTOOL = 'acltool'
    ARP_REFRESH = 'arp_refresh'
    CLAGD = 'clagd'
    DOCKER = 'docker'
    LEDMGRD = 'ledmgrd'
    LLDPD = 'lldpd'
    MSTPD = 'mstpd'
    NTP = 'ntp'
    POED = 'poed'
    PORTWD = 'portwd'
    PTMD = 'ptmd'
    PWMD = 'pwmd'
    RSYSLOG = 'rsyslog'
    SMOND = 'smond'
    SSH = 'ssh'
    SWITCHD = 'switchd'
    QUAGGA = 'quagga'
    FRR = 'frr'
    VXRD = 'vxrd'
    VXSND = 'vxsnd'
    WATCHDOG = 'wd_keepalive'
    MLX_SDK = 'sx_sdk'
    NEIGHMGRD = 'neighmgrd'
    NETQD = 'netqd'
    NETQAGENT = 'netq-agent'
    NETQNOTIFIER = 'netq-notifier'
    NETQSTATSPUSHD = 'netq-stats-pushd'
    CHASSISD = 'cumulus-chassisd'
    CHASSIS_SSH = 'cumulus-chassis-ssh'

class CentosRunningService(object):
    """Centos Specific Service name
    """
    NTP = 'ntpd'

class RhelRunningService(object):
    """RHEL Specific Service name
    """
    NTP = 'ntpd'

class ServiceStatus(object):
    '''Service status. Ensure string below is updated if this changes'''
    OK = 0
    WARNING = 1
    ERROR = 2
    FAIL = 3
    NA = 4


class ServiceStatusString(object):
    '''Service status string. Ensure this stays in sync with values'''
    OK = 'ok'
    WARNING = 'warning'
    ERROR = 'error'
    FAIL = 'fail'
    NA = 'n/a'


class RouteType(object):
    """Netlink route types imported from nlmanager
    """
    # Route types
    # /usr/include/linux/rtnetlink.h
    RTN_UNSPEC = 0x00  # Unknown broker.
    RTN_UNICAST = 0x01  # A gateway or direct broker.
    RTN_LOCAL = 0x02  # A local interface broker.
    RTN_BROADCAST = 0x03  # A local broadcast route(sent as a broadcast).
    RTN_ANYCAST = 0x04  # An anycast broker.
    RTN_MULTICAST = 0x05  # A multicast broker.
    RTN_BLACKHOLE = 0x06  # A silent packet dropping broker.
    RTN_UNREACHABLE = 0x07  # Packets dropped & host unreachable ICMPs sent
    RTN_PROHIBIT = 0x08  # Packets dropped and comm prohibited ICMPs sent
    RTN_THROW = 0x09  # continue lookup in next routing table as per ip rules
    RTN_NAT = 0x0A  # A network address translation rule.
    RTN_XRESOLVE = 0x0B  # Refer to an external resolver(not implemented).


class RouteTable(object):
    """Well known route table ids"""
    DEFAULT = 254
    LOCAL = 255
    RDNBR = 10


class StpPortState(object):
    '''Spanning tree port states as reported by netlink'''
    DISABLED = 0
    LISTENING = 1
    LEARNING = 2
    FORWARDING = 3
    BLOCKING = 4
    UNINIT = 5

